"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const global_middleware_1 = require("../../shared/middlewares/global.middleware");
const typedi_1 = __importDefault(require("typedi"));
const sourceControl_service_ee_1 = require("../../../../environments/sourceControl/sourceControl.service.ee");
const sourceControlPreferences_service_ee_1 = require("../../../../environments/sourceControl/sourceControlPreferences.service.ee");
const sourceControlHelper_ee_1 = require("../../../../environments/sourceControl/sourceControlHelper.ee");
module.exports = {
    pull: [
        (0, global_middleware_1.authorize)(['owner', 'member']),
        async (req, res) => {
            const sourceControlPreferencesService = typedi_1.default.get(sourceControlPreferences_service_ee_1.SourceControlPreferencesService);
            if (!(0, sourceControlHelper_ee_1.isSourceControlLicensed)()) {
                return res
                    .status(401)
                    .json({ status: 'Error', message: 'Source Control feature is not licensed' });
            }
            if (!sourceControlPreferencesService.isSourceControlConnected()) {
                return res
                    .status(400)
                    .json({ status: 'Error', message: 'Source Control is not connected to a repository' });
            }
            try {
                const sourceControlService = typedi_1.default.get(sourceControl_service_ee_1.SourceControlService);
                const result = await sourceControlService.pullWorkfolder({
                    force: req.body.force,
                    variables: req.body.variables,
                    userId: req.user.id,
                    importAfterPull: true,
                });
                if (result === null || result === void 0 ? void 0 : result.workflows) {
                    return res.status(200).send(result);
                }
                else {
                    return res.status(409).send(result);
                }
            }
            catch (error) {
                return res.status(400).send(error.message);
            }
        },
    ],
};
//# sourceMappingURL=sourceControl.handler.js.map