import type { SourceControlPreferences } from './types/sourceControlPreferences';
import { SourceControlGitService } from './sourceControlGit.service.ee';
import type { PushResult, StatusResult } from 'simple-git';
import type { ExportResult } from './types/exportResult';
import { SourceControlExportService } from './sourceControlExport.service.ee';
import type { ImportResult } from './types/importResult';
import type { SourceControlPushWorkFolder } from './types/sourceControlPushWorkFolder';
import type { SourceControllPullOptions } from './types/sourceControlPullWorkFolder';
import type { SourceControlledFile } from './types/sourceControlledFile';
import { SourceControlPreferencesService } from './sourceControlPreferences.service.ee';
import { SourceControlImportService } from './sourceControlImport.service.ee';
export declare class SourceControlService {
    private gitService;
    private sourceControlPreferencesService;
    private sourceControlExportService;
    private sourceControlImportService;
    private sshKeyName;
    private sshFolder;
    private gitFolder;
    constructor(gitService: SourceControlGitService, sourceControlPreferencesService: SourceControlPreferencesService, sourceControlExportService: SourceControlExportService, sourceControlImportService: SourceControlImportService);
    init(): Promise<void>;
    disconnect(options?: {
        keepKeyPair?: boolean;
    }): Promise<SourceControlPreferences>;
    initializeRepository(preferences: SourceControlPreferences): Promise<{
        branches: string[];
        currentBranch: string;
    }>;
    export(): Promise<{
        tags: ExportResult | undefined;
        credentials: ExportResult | undefined;
        variables: ExportResult | undefined;
        workflows: ExportResult | undefined;
    }>;
    import(options: SourceControllPullOptions): Promise<ImportResult | undefined>;
    getBranches(): Promise<{
        branches: string[];
        currentBranch: string;
    }>;
    setBranch(branch: string): Promise<{
        branches: string[];
        currentBranch: string;
    }>;
    resetWorkfolder(options: SourceControllPullOptions): Promise<ImportResult | undefined>;
    pushWorkfolder(options: SourceControlPushWorkFolder): Promise<PushResult | SourceControlledFile[]>;
    pullWorkfolder(options: SourceControllPullOptions): Promise<ImportResult | StatusResult | undefined>;
    stage(options: Pick<SourceControlPushWorkFolder, 'fileNames' | 'credentialIds' | 'workflowIds'>): Promise<{
        staged: string[];
    } | string>;
    unstage(): Promise<StatusResult | string>;
    status(): Promise<StatusResult>;
    private fileNameToSourceControlledFile;
    getStatus(): Promise<SourceControlledFile[]>;
}
