"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceControlService = void 0;
const typedi_1 = require("typedi");
const path_1 = __importDefault(require("path"));
const Db = __importStar(require("../../Db"));
const sourceControlHelper_ee_1 = require("./sourceControlHelper.ee");
const constants_1 = require("./constants");
const n8n_workflow_1 = require("n8n-workflow");
const sourceControlGit_service_ee_1 = require("./sourceControlGit.service.ee");
const n8n_core_1 = require("n8n-core");
const sourceControlExport_service_ee_1 = require("./sourceControlExport.service.ee");
const ResponseHelper_1 = require("../../ResponseHelper");
const sourceControlPreferences_service_ee_1 = require("./sourceControlPreferences.service.ee");
const fs_1 = require("fs");
const sourceControlImport_service_ee_1 = require("./sourceControlImport.service.ee");
let SourceControlService = exports.SourceControlService = class SourceControlService {
    constructor(gitService, sourceControlPreferencesService, sourceControlExportService, sourceControlImportService) {
        this.gitService = gitService;
        this.sourceControlPreferencesService = sourceControlPreferencesService;
        this.sourceControlExportService = sourceControlExportService;
        this.sourceControlImportService = sourceControlImportService;
        const userFolder = n8n_core_1.UserSettings.getUserN8nFolderPath();
        this.sshFolder = path_1.default.join(userFolder, constants_1.SOURCE_CONTROL_SSH_FOLDER);
        this.gitFolder = path_1.default.join(userFolder, constants_1.SOURCE_CONTROL_GIT_FOLDER);
        this.sshKeyName = path_1.default.join(this.sshFolder, constants_1.SOURCE_CONTROL_SSH_KEY_NAME);
    }
    async init() {
        this.gitService.resetService();
        (0, sourceControlHelper_ee_1.sourceControlFoldersExistCheck)([this.gitFolder, this.sshFolder]);
        await this.sourceControlPreferencesService.loadFromDbAndApplySourceControlPreferences();
        await this.gitService.initService({
            sourceControlPreferences: this.sourceControlPreferencesService.getPreferences(),
            gitFolder: this.gitFolder,
            sshKeyName: this.sshKeyName,
            sshFolder: this.sshFolder,
        });
    }
    async disconnect(options = {}) {
        try {
            await this.sourceControlPreferencesService.setPreferences({
                connected: false,
                branchName: '',
            });
            await this.sourceControlExportService.deleteRepositoryFolder();
            if (!options.keepKeyPair) {
                await this.sourceControlPreferencesService.deleteKeyPairFiles();
            }
            this.gitService.resetService();
            return this.sourceControlPreferencesService.sourceControlPreferences;
        }
        catch (error) {
            throw Error(`Failed to disconnect from source control: ${error.message}`);
        }
    }
    async initializeRepository(preferences) {
        var _a;
        if (!this.gitService.git) {
            await this.init();
        }
        n8n_workflow_1.LoggerProxy.debug('Initializing repository...');
        await this.gitService.initRepository(preferences);
        let getBranchesResult;
        try {
            getBranchesResult = await this.getBranches();
        }
        catch (error) {
            if (error.message.includes('Warning: Permanently added')) {
                n8n_workflow_1.LoggerProxy.debug('Added repository host to the list of known hosts. Retrying...');
                getBranchesResult = await this.getBranches();
            }
            else {
                throw error;
            }
        }
        if (getBranchesResult.branches.includes(preferences.branchName)) {
            await this.gitService.setBranch(preferences.branchName);
        }
        else {
            if (((_a = getBranchesResult.branches) === null || _a === void 0 ? void 0 : _a.length) === 0) {
                try {
                    (0, fs_1.writeFileSync)(path_1.default.join(this.gitFolder, '/README.md'), constants_1.SOURCE_CONTROL_README);
                    await this.gitService.stage(new Set(['README.md']));
                    await this.gitService.commit('Initial commit');
                    await this.gitService.push({
                        branch: preferences.branchName,
                        force: true,
                    });
                    getBranchesResult = await this.getBranches();
                }
                catch (fileError) {
                    n8n_workflow_1.LoggerProxy.error(`Failed to create initial commit: ${fileError.message}`);
                }
            }
            else {
                await this.sourceControlPreferencesService.setPreferences({
                    branchName: '',
                    connected: true,
                });
            }
        }
        return getBranchesResult;
    }
    async export() {
        const result = {
            credentials: undefined,
            tags: undefined,
            variables: undefined,
            workflows: undefined,
        };
        try {
            await this.sourceControlExportService.cleanWorkFolder();
            result.tags = await this.sourceControlExportService.exportTagsToWorkFolder();
            result.variables = await this.sourceControlExportService.exportVariablesToWorkFolder();
            result.workflows = await this.sourceControlExportService.exportWorkflowsToWorkFolder();
            result.credentials = await this.sourceControlExportService.exportCredentialsToWorkFolder();
        }
        catch (error) {
            throw new ResponseHelper_1.BadRequestError(error.message);
        }
        return result;
    }
    async import(options) {
        try {
            return await this.sourceControlImportService.importFromWorkFolder(options);
        }
        catch (error) {
            throw new ResponseHelper_1.BadRequestError(error.message);
        }
    }
    async getBranches() {
        await this.gitService.fetch();
        return this.gitService.getBranches();
    }
    async setBranch(branch) {
        await this.sourceControlPreferencesService.setPreferences({
            branchName: branch,
            connected: (branch === null || branch === void 0 ? void 0 : branch.length) > 0,
        });
        return this.gitService.setBranch(branch);
    }
    async resetWorkfolder(options) {
        const currentBranch = await this.gitService.getCurrentBranch();
        await this.sourceControlExportService.cleanWorkFolder();
        await this.gitService.resetBranch({
            hard: true,
            target: currentBranch.remote,
        });
        await this.gitService.pull();
        if (options.importAfterPull) {
            return this.import(options);
        }
        return;
    }
    async pushWorkfolder(options) {
        var _a, _b;
        if (this.sourceControlPreferencesService.isBranchReadOnly()) {
            throw new ResponseHelper_1.BadRequestError('Cannot push onto read-only branch.');
        }
        if (!options.skipDiff) {
            const diffResult = await this.getStatus();
            const possibleConflicts = diffResult === null || diffResult === void 0 ? void 0 : diffResult.filter((file) => file.conflict);
            if ((possibleConflicts === null || possibleConflicts === void 0 ? void 0 : possibleConflicts.length) > 0 && options.force !== true) {
                await this.unstage();
                return diffResult;
            }
        }
        await this.unstage();
        await this.stage(options);
        await this.gitService.commit((_a = options.message) !== null && _a !== void 0 ? _a : 'Updated Workfolder');
        return this.gitService.push({
            branch: this.sourceControlPreferencesService.getBranchName(),
            force: (_b = options.force) !== null && _b !== void 0 ? _b : false,
        });
    }
    async pullWorkfolder(options) {
        await this.resetWorkfolder({
            importAfterPull: false,
            userId: options.userId,
            force: false,
        });
        await this.export();
        const status = await this.gitService.status();
        if (status.modified.length > 0 && options.force !== true) {
            return status;
        }
        await this.resetWorkfolder({ ...options, importAfterPull: false });
        if (options.importAfterPull) {
            return this.import(options);
        }
        return;
    }
    async stage(options) {
        const { fileNames, credentialIds, workflowIds } = options;
        const status = await this.gitService.status();
        let mergedFileNames = new Set();
        fileNames === null || fileNames === void 0 ? void 0 : fileNames.forEach((e) => mergedFileNames.add(e));
        credentialIds === null || credentialIds === void 0 ? void 0 : credentialIds.forEach((e) => mergedFileNames.add(this.sourceControlExportService.getCredentialsPath(e)));
        workflowIds === null || workflowIds === void 0 ? void 0 : workflowIds.forEach((e) => mergedFileNames.add(this.sourceControlExportService.getWorkflowPath(e)));
        if (mergedFileNames.size === 0) {
            mergedFileNames = new Set([
                ...status.not_added,
                ...status.created,
                ...status.modified,
            ]);
        }
        const deletedFiles = new Set(status.deleted);
        deletedFiles.forEach((e) => mergedFileNames.delete(e));
        await this.unstage();
        const stageResult = await this.gitService.stage(mergedFileNames, deletedFiles);
        if (!stageResult) {
            const statusResult = await this.gitService.status();
            return { staged: statusResult.staged };
        }
        return stageResult;
    }
    async unstage() {
        const stageResult = await this.gitService.resetBranch();
        if (!stageResult) {
            return this.gitService.status();
        }
        return stageResult;
    }
    async status() {
        return this.gitService.status();
    }
    async fileNameToSourceControlledFile(fileName, location, statusResult) {
        let id = undefined;
        let name = '';
        let conflict = false;
        let status = 'unknown';
        let type = 'file';
        if (statusResult.not_added.find((e) => e === fileName))
            status = 'new';
        else if (statusResult.conflicted.find((e) => e === fileName)) {
            status = 'conflicted';
            conflict = true;
        }
        else if (statusResult.created.find((e) => e === fileName))
            status = 'created';
        else if (statusResult.deleted.find((e) => e === fileName))
            status = 'deleted';
        else if (statusResult.modified.find((e) => e === fileName))
            status = 'modified';
        if (fileName.startsWith(constants_1.SOURCE_CONTROL_WORKFLOW_EXPORT_FOLDER)) {
            type = 'workflow';
            if (status === 'deleted') {
                id = fileName
                    .replace(constants_1.SOURCE_CONTROL_WORKFLOW_EXPORT_FOLDER, '')
                    .replace(/[\/,\\]/, '')
                    .replace('.json', '');
                if (location === 'remote') {
                    const existingWorkflow = await Db.collections.Workflow.find({
                        where: { id },
                    });
                    if ((existingWorkflow === null || existingWorkflow === void 0 ? void 0 : existingWorkflow.length) > 0) {
                        name = existingWorkflow[0].name;
                    }
                }
                else {
                    name = '(deleted)';
                }
            }
            else {
                const workflow = await this.sourceControlExportService.getWorkflowFromFile(fileName);
                if (!(workflow === null || workflow === void 0 ? void 0 : workflow.id)) {
                    if (location === 'local') {
                        return;
                    }
                    id = fileName
                        .replace(constants_1.SOURCE_CONTROL_WORKFLOW_EXPORT_FOLDER + '/', '')
                        .replace('.json', '');
                    status = 'created';
                }
                else {
                    id = workflow.id;
                    name = workflow.name;
                }
            }
        }
        if (fileName.startsWith(constants_1.SOURCE_CONTROL_CREDENTIAL_EXPORT_FOLDER)) {
            type = 'credential';
            if (status === 'deleted') {
                id = fileName
                    .replace(constants_1.SOURCE_CONTROL_CREDENTIAL_EXPORT_FOLDER, '')
                    .replace(/[\/,\\]/, '')
                    .replace('.json', '');
                if (location === 'remote') {
                    const existingCredential = await Db.collections.Credentials.find({
                        where: { id },
                    });
                    if ((existingCredential === null || existingCredential === void 0 ? void 0 : existingCredential.length) > 0) {
                        name = existingCredential[0].name;
                    }
                }
                else {
                    name = '(deleted)';
                }
            }
            else {
                const credential = await this.sourceControlExportService.getCredentialFromFile(fileName);
                if (!(credential === null || credential === void 0 ? void 0 : credential.id)) {
                    if (location === 'local') {
                        return;
                    }
                    id = fileName
                        .replace(constants_1.SOURCE_CONTROL_CREDENTIAL_EXPORT_FOLDER + '/', '')
                        .replace('.json', '');
                    status = 'created';
                }
                else {
                    id = credential.id;
                    name = credential.name;
                }
            }
        }
        if (fileName.startsWith(constants_1.SOURCE_CONTROL_VARIABLES_EXPORT_FILE)) {
            id = 'variables';
            name = 'variables';
            type = 'variables';
        }
        if (fileName.startsWith(constants_1.SOURCE_CONTROL_TAGS_EXPORT_FILE)) {
            id = 'tags';
            name = 'tags';
            type = 'tags';
        }
        if (!id)
            return;
        return {
            file: fileName,
            id,
            name,
            type,
            status,
            location,
            conflict,
        };
    }
    async getStatus() {
        var _a, _b;
        await this.export();
        await this.stage({});
        await this.gitService.fetch();
        const sourceControlledFiles = [];
        const diffRemote = await this.gitService.diffRemote();
        const diffLocal = await this.gitService.diffLocal();
        const status = await this.gitService.status();
        await Promise.all([
            ...((_a = diffRemote === null || diffRemote === void 0 ? void 0 : diffRemote.files.map(async (e) => {
                const resolvedFile = await this.fileNameToSourceControlledFile(e.file, 'remote', status);
                if (resolvedFile) {
                    sourceControlledFiles.push(resolvedFile);
                }
            })) !== null && _a !== void 0 ? _a : []),
            ...((_b = diffLocal === null || diffLocal === void 0 ? void 0 : diffLocal.files.map(async (e) => {
                const resolvedFile = await this.fileNameToSourceControlledFile(e.file, 'local', status);
                if (resolvedFile) {
                    sourceControlledFiles.push(resolvedFile);
                }
            })) !== null && _b !== void 0 ? _b : []),
        ]);
        sourceControlledFiles.forEach((e, index, array) => {
            const similarItems = array.filter((f) => f.type === e.type && (f.file === e.file || f.id === e.id));
            if (similarItems.length > 1) {
                similarItems.forEach((item) => {
                    item.conflict = true;
                });
            }
        });
        return sourceControlledFiles;
    }
};
exports.SourceControlService = SourceControlService = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [sourceControlGit_service_ee_1.SourceControlGitService,
        sourceControlPreferences_service_ee_1.SourceControlPreferencesService,
        sourceControlExport_service_ee_1.SourceControlExportService,
        sourceControlImport_service_ee_1.SourceControlImportService])
], SourceControlService);
//# sourceMappingURL=sourceControl.service.ee.js.map