"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceControlExportService = void 0;
const typedi_1 = require("typedi");
const path_1 = __importDefault(require("path"));
const constants_1 = require("./constants");
const Db = __importStar(require("../../Db"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const n8n_workflow_1 = require("n8n-workflow");
const promises_1 = require("fs/promises");
const n8n_core_1 = require("n8n-core");
const sourceControlHelper_ee_1 = require("./sourceControlHelper.ee");
let SourceControlExportService = exports.SourceControlExportService = class SourceControlExportService {
    constructor() {
        this.replaceCredentialData = (data) => {
            var _a;
            for (const [key] of Object.entries(data)) {
                try {
                    if (data[key] === null) {
                        delete data[key];
                    }
                    else if (typeof data[key] === 'object') {
                        data[key] = this.replaceCredentialData(data[key]);
                    }
                    else if (typeof data[key] === 'string') {
                        data[key] = ((_a = data[key]) === null || _a === void 0 ? void 0 : _a.startsWith('={{')) ? data[key] : '';
                    }
                    else if (typeof data[key] === 'number') {
                        continue;
                    }
                }
                catch (error) {
                    n8n_workflow_1.LoggerProxy.error(`Failed to sanitize credential data: ${error.message}`);
                    throw error;
                }
            }
            return data;
        };
        const userFolder = n8n_core_1.UserSettings.getUserN8nFolderPath();
        this.gitFolder = path_1.default.join(userFolder, constants_1.SOURCE_CONTROL_GIT_FOLDER);
        this.workflowExportFolder = path_1.default.join(this.gitFolder, constants_1.SOURCE_CONTROL_WORKFLOW_EXPORT_FOLDER);
        this.credentialExportFolder = path_1.default.join(this.gitFolder, constants_1.SOURCE_CONTROL_CREDENTIAL_EXPORT_FOLDER);
    }
    getWorkflowPath(workflowId) {
        return path_1.default.join(this.workflowExportFolder, `${workflowId}.json`);
    }
    getCredentialsPath(credentialsId) {
        return path_1.default.join(this.credentialExportFolder, `${credentialsId}.json`);
    }
    getTagsPath() {
        return path_1.default.join(this.gitFolder, constants_1.SOURCE_CONTROL_TAGS_EXPORT_FILE);
    }
    getVariablesPath() {
        return path_1.default.join(this.gitFolder, constants_1.SOURCE_CONTROL_VARIABLES_EXPORT_FILE);
    }
    async getWorkflowFromFile(filePath, root = this.gitFolder) {
        try {
            const importedWorkflow = (0, n8n_workflow_1.jsonParse)(await (0, promises_1.readFile)(path_1.default.join(root, filePath), { encoding: 'utf8' }));
            return importedWorkflow;
        }
        catch (error) {
            return undefined;
        }
    }
    async getCredentialFromFile(filePath, root = this.gitFolder) {
        try {
            const credential = (0, n8n_workflow_1.jsonParse)(await (0, promises_1.readFile)(path_1.default.join(root, filePath), { encoding: 'utf8' }));
            return credential;
        }
        catch (error) {
            return undefined;
        }
    }
    async cleanWorkFolder() {
        try {
            const workflowFiles = await (0, fast_glob_1.default)('*.json', {
                cwd: this.workflowExportFolder,
                absolute: true,
            });
            const credentialFiles = await (0, fast_glob_1.default)('*.json', {
                cwd: this.credentialExportFolder,
                absolute: true,
            });
            const variablesFile = await (0, fast_glob_1.default)(constants_1.SOURCE_CONTROL_VARIABLES_EXPORT_FILE, {
                cwd: this.gitFolder,
                absolute: true,
            });
            const tagsFile = await (0, fast_glob_1.default)(constants_1.SOURCE_CONTROL_TAGS_EXPORT_FILE, {
                cwd: this.gitFolder,
                absolute: true,
            });
            await Promise.all(tagsFile.map(async (e) => (0, promises_1.rm)(e)));
            await Promise.all(variablesFile.map(async (e) => (0, promises_1.rm)(e)));
            await Promise.all(workflowFiles.map(async (e) => (0, promises_1.rm)(e)));
            await Promise.all(credentialFiles.map(async (e) => (0, promises_1.rm)(e)));
            n8n_workflow_1.LoggerProxy.debug('Cleaned work folder.');
        }
        catch (error) {
            n8n_workflow_1.LoggerProxy.error(`Failed to clean work folder: ${error.message}`);
        }
    }
    async deleteRepositoryFolder() {
        try {
            await (0, promises_1.rm)(this.gitFolder, { recursive: true });
        }
        catch (error) {
            n8n_workflow_1.LoggerProxy.error(`Failed to delete work folder: ${error.message}`);
        }
    }
    async rmDeletedWorkflowsFromExportFolder(workflowsToBeExported) {
        const sharedWorkflowsFileNames = new Set(workflowsToBeExported.map((e) => { var _a; return this.getWorkflowPath((_a = e === null || e === void 0 ? void 0 : e.workflow) === null || _a === void 0 ? void 0 : _a.name); }));
        const existingWorkflowsInFolder = new Set(await (0, fast_glob_1.default)('*.json', {
            cwd: this.workflowExportFolder,
            absolute: true,
        }));
        const deletedWorkflows = new Set(existingWorkflowsInFolder);
        for (const elem of sharedWorkflowsFileNames) {
            deletedWorkflows.delete(elem);
        }
        try {
            await Promise.all([...deletedWorkflows].map(async (e) => (0, promises_1.rm)(e)));
        }
        catch (error) {
            n8n_workflow_1.LoggerProxy.error(`Failed to delete workflows from work folder: ${error.message}`);
        }
        return deletedWorkflows;
    }
    async writeExportableWorkflowsToExportFolder(workflowsToBeExported) {
        await Promise.all(workflowsToBeExported.map(async (e) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            if (!e.workflow) {
                n8n_workflow_1.LoggerProxy.debug(`Found no corresponding workflow ${(_a = e.workflowId) !== null && _a !== void 0 ? _a : 'unknown'}, skipping export`);
                return;
            }
            const fileName = this.getWorkflowPath((_b = e.workflow) === null || _b === void 0 ? void 0 : _b.id);
            const sanitizedWorkflow = {
                active: (_c = e.workflow) === null || _c === void 0 ? void 0 : _c.active,
                id: (_d = e.workflow) === null || _d === void 0 ? void 0 : _d.id,
                name: (_e = e.workflow) === null || _e === void 0 ? void 0 : _e.name,
                nodes: (_f = e.workflow) === null || _f === void 0 ? void 0 : _f.nodes,
                connections: (_g = e.workflow) === null || _g === void 0 ? void 0 : _g.connections,
                settings: (_h = e.workflow) === null || _h === void 0 ? void 0 : _h.settings,
                triggerCount: (_j = e.workflow) === null || _j === void 0 ? void 0 : _j.triggerCount,
                owner: e.user.email,
                versionId: (_k = e.workflow) === null || _k === void 0 ? void 0 : _k.versionId,
            };
            n8n_workflow_1.LoggerProxy.debug(`Writing workflow ${e.workflowId} to ${fileName}`);
            return (0, promises_1.writeFile)(fileName, JSON.stringify(sanitizedWorkflow, null, 2));
        }));
    }
    async exportWorkflowsToWorkFolder() {
        try {
            (0, sourceControlHelper_ee_1.sourceControlFoldersExistCheck)([this.workflowExportFolder]);
            const sharedWorkflows = await Db.collections.SharedWorkflow.find({
                relations: ['workflow', 'role', 'user'],
                where: {
                    role: {
                        name: 'owner',
                        scope: 'workflow',
                    },
                },
            });
            const removedFiles = await this.rmDeletedWorkflowsFromExportFolder(sharedWorkflows);
            await this.writeExportableWorkflowsToExportFolder(sharedWorkflows);
            return {
                count: sharedWorkflows.length,
                folder: this.workflowExportFolder,
                files: sharedWorkflows.map((e) => {
                    var _a, _b;
                    return ({
                        id: (_a = e === null || e === void 0 ? void 0 : e.workflow) === null || _a === void 0 ? void 0 : _a.id,
                        name: this.getWorkflowPath((_b = e === null || e === void 0 ? void 0 : e.workflow) === null || _b === void 0 ? void 0 : _b.name),
                    });
                }),
                removedFiles: [...removedFiles],
            };
        }
        catch (error) {
            throw Error(`Failed to export workflows to work folder: ${error.message}`);
        }
    }
    async exportVariablesToWorkFolder() {
        try {
            (0, sourceControlHelper_ee_1.sourceControlFoldersExistCheck)([this.gitFolder]);
            const variables = await Db.collections.Variables.find();
            if (variables.length === 0) {
                return {
                    count: 0,
                    folder: this.gitFolder,
                    files: [],
                };
            }
            const fileName = this.getVariablesPath();
            const sanitizedVariables = variables.map((e) => ({ ...e, value: '' }));
            await (0, promises_1.writeFile)(fileName, JSON.stringify(sanitizedVariables, null, 2));
            return {
                count: sanitizedVariables.length,
                folder: this.gitFolder,
                files: [
                    {
                        id: '',
                        name: fileName,
                    },
                ],
            };
        }
        catch (error) {
            throw Error(`Failed to export variables to work folder: ${error.message}`);
        }
    }
    async exportTagsToWorkFolder() {
        try {
            (0, sourceControlHelper_ee_1.sourceControlFoldersExistCheck)([this.gitFolder]);
            const tags = await Db.collections.Tag.find();
            if (tags.length === 0) {
                return {
                    count: 0,
                    folder: this.gitFolder,
                    files: [],
                };
            }
            const mappings = await Db.collections.WorkflowTagMapping.find();
            const fileName = this.getTagsPath();
            await (0, promises_1.writeFile)(fileName, JSON.stringify({
                tags: tags.map((tag) => ({ id: tag.id, name: tag.name })),
                mappings,
            }, null, 2));
            return {
                count: tags.length,
                folder: this.gitFolder,
                files: [
                    {
                        id: '',
                        name: fileName,
                    },
                ],
            };
        }
        catch (error) {
            throw Error(`Failed to export variables to work folder: ${error.message}`);
        }
    }
    async exportCredentialsToWorkFolder() {
        try {
            (0, sourceControlHelper_ee_1.sourceControlFoldersExistCheck)([this.credentialExportFolder]);
            const sharedCredentials = await Db.collections.SharedCredentials.find({
                relations: ['credentials', 'role', 'user'],
            });
            const encryptionKey = await n8n_core_1.UserSettings.getEncryptionKey();
            await Promise.all(sharedCredentials.map(async (sharedCredential) => {
                const { name, type, nodesAccess, data, id } = sharedCredential.credentials;
                const credentialObject = new n8n_core_1.Credentials({ id, name }, type, nodesAccess, data);
                const plainData = credentialObject.getData(encryptionKey);
                const sanitizedData = this.replaceCredentialData(plainData);
                const fileName = path_1.default.join(this.credentialExportFolder, `${sharedCredential.credentials.id}.json`);
                const sanitizedCredential = {
                    id: sharedCredential.credentials.id,
                    name: sharedCredential.credentials.name,
                    type: sharedCredential.credentials.type,
                    data: sanitizedData,
                    nodesAccess: sharedCredential.credentials.nodesAccess,
                };
                n8n_workflow_1.LoggerProxy.debug(`Writing credential ${sharedCredential.credentials.id} to ${fileName}`);
                return (0, promises_1.writeFile)(fileName, JSON.stringify(sanitizedCredential, null, 2));
            }));
            return {
                count: sharedCredentials.length,
                folder: this.credentialExportFolder,
                files: sharedCredentials.map((e) => ({
                    id: e.credentials.id,
                    name: path_1.default.join(this.credentialExportFolder, `${e.credentials.name}.json`),
                })),
            };
        }
        catch (error) {
            throw Error(`Failed to export credentials to work folder: ${error.message}`);
        }
    }
};
exports.SourceControlExportService = SourceControlExportService = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [])
], SourceControlExportService);
//# sourceMappingURL=sourceControlExport.service.ee.js.map