import type { CommitResult, DiffResult, FetchResult, PullResult, PushResult, SimpleGit, StatusResult } from 'simple-git';
import type { SourceControlPreferences } from './types/sourceControlPreferences';
export declare class SourceControlGitService {
    git: SimpleGit | null;
    private gitOptions;
    preInitCheck(): boolean;
    initService(options: {
        sourceControlPreferences: SourceControlPreferences;
        gitFolder: string;
        sshFolder: string;
        sshKeyName: string;
    }): Promise<void>;
    resetService(): void;
    checkRepositorySetup(): Promise<boolean>;
    hasRemote(remote: string): Promise<boolean>;
    initRepository(sourceControlPreferences: Pick<SourceControlPreferences, 'repositoryUrl' | 'authorEmail' | 'authorName' | 'branchName' | 'initRepo'>): Promise<void>;
    getBranches(): Promise<{
        branches: string[];
        currentBranch: string;
    }>;
    setBranch(branch: string): Promise<{
        branches: string[];
        currentBranch: string;
    }>;
    fetch(): Promise<FetchResult>;
    getCurrentBranch(): Promise<{
        current: string;
        remote: string;
    }>;
    diff(options?: {
        target?: string;
        dots?: '..' | '...';
    }): Promise<DiffResult>;
    diffRemote(): Promise<DiffResult | undefined>;
    diffLocal(): Promise<DiffResult | undefined>;
    pull(options?: {
        ffOnly: boolean;
    }): Promise<PullResult>;
    push(options?: {
        force: boolean;
        branch: string;
    }): Promise<PushResult>;
    stage(files: Set<string>, deletedFiles?: Set<string>): Promise<string>;
    resetBranch(options?: {
        hard?: boolean;
        target: string;
    }): Promise<string>;
    commit(message: string): Promise<CommitResult>;
    status(): Promise<StatusResult>;
}
