"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSshKeyPair = exports.isSourceControlLicensed = exports.sourceControlFoldersExistCheck = void 0;
const typedi_1 = __importDefault(require("typedi"));
const License_1 = require("../../License");
const crypto_1 = require("crypto");
const sshpk_1 = __importDefault(require("sshpk"));
const fs_1 = require("fs");
const n8n_workflow_1 = require("n8n-workflow");
const constants_1 = require("./constants");
function sourceControlFoldersExistCheck(folders) {
    folders.forEach((folder) => {
        try {
            (0, fs_1.accessSync)(folder, fs_1.constants.F_OK);
        }
        catch {
            try {
                (0, fs_1.mkdirSync)(folder);
            }
            catch (error) {
                n8n_workflow_1.LoggerProxy.error(error.message);
            }
        }
    });
}
exports.sourceControlFoldersExistCheck = sourceControlFoldersExistCheck;
function isSourceControlLicensed() {
    const license = typedi_1.default.get(License_1.License);
    return license.isSourceControlLicensed();
}
exports.isSourceControlLicensed = isSourceControlLicensed;
function generateSshKeyPair(keyType = 'ed25519') {
    const keyPair = {
        publicKey: '',
        privateKey: '',
    };
    let generatedKeyPair;
    switch (keyType) {
        case 'ed25519':
            generatedKeyPair = (0, crypto_1.generateKeyPairSync)('ed25519', {
                privateKeyEncoding: { format: 'pem', type: 'pkcs8' },
                publicKeyEncoding: { format: 'pem', type: 'spki' },
            });
            break;
        case 'rsa':
            generatedKeyPair = (0, crypto_1.generateKeyPairSync)('rsa', {
                modulusLength: 4096,
                publicKeyEncoding: {
                    type: 'spki',
                    format: 'pem',
                },
                privateKeyEncoding: {
                    type: 'pkcs8',
                    format: 'pem',
                },
            });
            break;
    }
    const keyPublic = sshpk_1.default.parseKey(generatedKeyPair.publicKey, 'pem');
    keyPublic.comment = constants_1.SOURCE_CONTROL_GIT_KEY_COMMENT;
    keyPair.publicKey = keyPublic.toString('ssh');
    const keyPrivate = sshpk_1.default.parsePrivateKey(generatedKeyPair.privateKey, 'pem');
    keyPrivate.comment = constants_1.SOURCE_CONTROL_GIT_KEY_COMMENT;
    keyPair.privateKey = keyPrivate.toString('ssh-private');
    return {
        privateKey: keyPair.privateKey,
        publicKey: keyPair.publicKey,
    };
}
exports.generateSshKeyPair = generateSshKeyPair;
//# sourceMappingURL=sourceControlHelper.ee.js.map