"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceControlImportService = void 0;
const typedi_1 = __importStar(require("typedi"));
const path_1 = __importDefault(require("path"));
const constants_1 = require("./constants");
const Db = __importStar(require("../../Db"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const n8n_workflow_1 = require("n8n-workflow");
const promises_1 = require("fs/promises");
const n8n_core_1 = require("n8n-core");
const SharedWorkflow_1 = require("../../databases/entities/SharedWorkflow");
const CredentialsEntity_1 = require("../../databases/entities/CredentialsEntity");
const Variables_1 = require("../../databases/entities/Variables");
const BaseCommand_1 = require("../../commands/BaseCommand");
const SharedCredentials_1 = require("../../databases/entities/SharedCredentials");
const WorkflowEntity_1 = require("../../databases/entities/WorkflowEntity");
const WorkflowTagMapping_1 = require("../../databases/entities/WorkflowTagMapping");
const TagEntity_1 = require("../../databases/entities/TagEntity");
const ActiveWorkflowRunner_1 = require("../../ActiveWorkflowRunner");
const typeorm_1 = require("typeorm");
const ResponseHelper_1 = require("../../ResponseHelper");
let SourceControlImportService = exports.SourceControlImportService = class SourceControlImportService {
    constructor() {
        const userFolder = n8n_core_1.UserSettings.getUserN8nFolderPath();
        this.gitFolder = path_1.default.join(userFolder, constants_1.SOURCE_CONTROL_GIT_FOLDER);
        this.workflowExportFolder = path_1.default.join(this.gitFolder, constants_1.SOURCE_CONTROL_WORKFLOW_EXPORT_FOLDER);
        this.credentialExportFolder = path_1.default.join(this.gitFolder, constants_1.SOURCE_CONTROL_CREDENTIAL_EXPORT_FOLDER);
    }
    async getOwnerGlobalRole() {
        const ownerCredentiallRole = await Db.collections.Role.findOne({
            where: { name: 'owner', scope: 'global' },
        });
        if (!ownerCredentiallRole) {
            throw new Error(`Failed to find owner. ${BaseCommand_1.UM_FIX_INSTRUCTION}`);
        }
        return ownerCredentiallRole;
    }
    async getOwnerCredentialRole() {
        const ownerCredentiallRole = await Db.collections.Role.findOne({
            where: { name: 'owner', scope: 'credential' },
        });
        if (!ownerCredentiallRole) {
            throw new Error(`Failed to find owner. ${BaseCommand_1.UM_FIX_INSTRUCTION}`);
        }
        return ownerCredentiallRole;
    }
    async getOwnerWorkflowRole() {
        const ownerWorkflowRole = await Db.collections.Role.findOne({
            where: { name: 'owner', scope: 'workflow' },
        });
        if (!ownerWorkflowRole) {
            throw new Error(`Failed to find owner workflow role. ${BaseCommand_1.UM_FIX_INSTRUCTION}`);
        }
        return ownerWorkflowRole;
    }
    async importCredentialsFromFiles(userId) {
        const credentialFiles = await (0, fast_glob_1.default)('*.json', {
            cwd: this.credentialExportFolder,
            absolute: true,
        });
        const existingCredentials = await Db.collections.Credentials.find();
        const ownerCredentialRole = await this.getOwnerCredentialRole();
        const ownerGlobalRole = await this.getOwnerGlobalRole();
        const encryptionKey = await n8n_core_1.UserSettings.getEncryptionKey();
        let importCredentialsResult = [];
        await Db.transaction(async (transactionManager) => {
            importCredentialsResult = await Promise.all(credentialFiles.map(async (file) => {
                n8n_workflow_1.LoggerProxy.debug(`Importing credentials file ${file}`);
                const credential = (0, n8n_workflow_1.jsonParse)(await (0, promises_1.readFile)(file, { encoding: 'utf8' }));
                const existingCredential = existingCredentials.find((e) => e.id === credential.id && e.type === credential.type);
                const sharedOwner = await Db.collections.SharedCredentials.findOne({
                    select: ['userId'],
                    where: {
                        credentialsId: credential.id,
                        roleId: (0, typeorm_1.In)([ownerCredentialRole.id, ownerGlobalRole.id]),
                    },
                });
                const { name, type, data, id, nodesAccess } = credential;
                const newCredentialObject = new n8n_core_1.Credentials({ id, name }, type, []);
                if (existingCredential === null || existingCredential === void 0 ? void 0 : existingCredential.data) {
                    newCredentialObject.data = existingCredential.data;
                }
                else {
                    newCredentialObject.setData(data, encryptionKey);
                }
                newCredentialObject.nodesAccess = nodesAccess || (existingCredential === null || existingCredential === void 0 ? void 0 : existingCredential.nodesAccess) || [];
                n8n_workflow_1.LoggerProxy.debug(`Updating credential id ${newCredentialObject.id}`);
                await transactionManager.upsert(CredentialsEntity_1.CredentialsEntity, newCredentialObject, ['id']);
                if (!sharedOwner) {
                    const newSharedCredential = new SharedCredentials_1.SharedCredentials();
                    newSharedCredential.credentialsId = newCredentialObject.id;
                    newSharedCredential.userId = userId;
                    newSharedCredential.roleId = ownerGlobalRole.id;
                    await transactionManager.upsert(SharedCredentials_1.SharedCredentials, { ...newSharedCredential }, [
                        'credentialsId',
                        'userId',
                    ]);
                }
                return {
                    id: newCredentialObject.id,
                    name: newCredentialObject.name,
                    type: newCredentialObject.type,
                };
            }));
        });
        return importCredentialsResult.filter((e) => e !== undefined);
    }
    async importVariablesFromFile(valueOverrides) {
        const variablesFile = await (0, fast_glob_1.default)(constants_1.SOURCE_CONTROL_VARIABLES_EXPORT_FILE, {
            cwd: this.gitFolder,
            absolute: true,
        });
        const result = { imported: [] };
        if (variablesFile.length > 0) {
            n8n_workflow_1.LoggerProxy.debug(`Importing variables from file ${variablesFile[0]}`);
            const importedVariables = (0, n8n_workflow_1.jsonParse)(await (0, promises_1.readFile)(variablesFile[0], { encoding: 'utf8' }), { fallbackValue: [] });
            const overriddenKeys = Object.keys(valueOverrides !== null && valueOverrides !== void 0 ? valueOverrides : {});
            for (const variable of importedVariables) {
                if (!variable.key) {
                    continue;
                }
                if (variable.value === '') {
                    variable.value = undefined;
                }
                if (overriddenKeys.includes(variable.key) && valueOverrides) {
                    variable.value = valueOverrides[variable.key];
                    overriddenKeys.splice(overriddenKeys.indexOf(variable.key), 1);
                }
                try {
                    await Db.collections.Variables.upsert({ ...variable }, ['id']);
                }
                catch (errorUpsert) {
                    if ((0, ResponseHelper_1.isUniqueConstraintError)(errorUpsert)) {
                        n8n_workflow_1.LoggerProxy.debug(`Variable ${variable.key} already exists, updating instead`);
                        try {
                            await Db.collections.Variables.update({ key: variable.key }, { ...variable });
                        }
                        catch (errorUpdate) {
                            n8n_workflow_1.LoggerProxy.debug(`Failed to update variable ${variable.key}, skipping`);
                            n8n_workflow_1.LoggerProxy.debug(errorUpdate.message);
                        }
                    }
                }
                finally {
                    result.imported.push(variable.key);
                }
            }
            if (overriddenKeys.length > 0 && valueOverrides) {
                for (const key of overriddenKeys) {
                    result.imported.push(key);
                    const newVariable = new Variables_1.Variables({ key, value: valueOverrides[key] });
                    await Db.collections.Variables.save(newVariable);
                }
            }
        }
        return result;
    }
    async importTagsFromFile() {
        const tagsFile = await (0, fast_glob_1.default)(constants_1.SOURCE_CONTROL_TAGS_EXPORT_FILE, {
            cwd: this.gitFolder,
            absolute: true,
        });
        if (tagsFile.length > 0) {
            n8n_workflow_1.LoggerProxy.debug(`Importing tags from file ${tagsFile[0]}`);
            const mappedTags = (0, n8n_workflow_1.jsonParse)(await (0, promises_1.readFile)(tagsFile[0], { encoding: 'utf8' }), { fallbackValue: { tags: [], mappings: [] } });
            const existingWorkflowIds = new Set((await Db.collections.Workflow.find({
                select: ['id'],
            })).map((e) => e.id));
            await Db.transaction(async (transactionManager) => {
                await Promise.all(mappedTags.tags.map(async (tag) => {
                    await transactionManager.upsert(TagEntity_1.TagEntity, {
                        ...tag,
                    }, {
                        skipUpdateIfNoValuesChanged: true,
                        conflictPaths: { id: true },
                    });
                }));
                await Promise.all(mappedTags.mappings.map(async (mapping) => {
                    if (!existingWorkflowIds.has(String(mapping.workflowId)))
                        return;
                    await transactionManager.upsert(WorkflowTagMapping_1.WorkflowTagMapping, { tagId: String(mapping.tagId), workflowId: String(mapping.workflowId) }, {
                        skipUpdateIfNoValuesChanged: true,
                        conflictPaths: { tagId: true, workflowId: true },
                    });
                }));
            });
            return mappedTags;
        }
        return { tags: [], mappings: [] };
    }
    async importWorkflowsFromFiles(userId) {
        const workflowFiles = await (0, fast_glob_1.default)('*.json', {
            cwd: this.workflowExportFolder,
            absolute: true,
        });
        const existingWorkflows = await Db.collections.Workflow.find({
            select: ['id', 'name', 'active', 'versionId'],
        });
        const ownerWorkflowRole = await this.getOwnerWorkflowRole();
        const workflowRunner = typedi_1.default.get(ActiveWorkflowRunner_1.ActiveWorkflowRunner);
        let importWorkflowsResult = new Array();
        await Db.transaction(async (transactionManager) => {
            importWorkflowsResult = await Promise.all(workflowFiles.map(async (file) => {
                var _a, _b, _c, _d, _e, _f, _g, _h;
                n8n_workflow_1.LoggerProxy.debug(`Parsing workflow file ${file}`);
                const importedWorkflow = (0, n8n_workflow_1.jsonParse)(await (0, promises_1.readFile)(file, { encoding: 'utf8' }));
                const existingWorkflow = existingWorkflows.find((e) => e.id === importedWorkflow.id);
                if ((existingWorkflow === null || existingWorkflow === void 0 ? void 0 : existingWorkflow.versionId) === importedWorkflow.versionId) {
                    n8n_workflow_1.LoggerProxy.debug(`Skipping import of workflow ${(_a = importedWorkflow.id) !== null && _a !== void 0 ? _a : 'n/a'} - versionId is up to date`);
                    return {
                        id: (_b = importedWorkflow.id) !== null && _b !== void 0 ? _b : 'n/a',
                        name: 'skipped',
                    };
                }
                n8n_workflow_1.LoggerProxy.debug(`Importing workflow ${(_c = importedWorkflow.id) !== null && _c !== void 0 ? _c : 'n/a'}`);
                importedWorkflow.active = (_d = existingWorkflow === null || existingWorkflow === void 0 ? void 0 : existingWorkflow.active) !== null && _d !== void 0 ? _d : false;
                n8n_workflow_1.LoggerProxy.debug(`Updating workflow id ${(_e = importedWorkflow.id) !== null && _e !== void 0 ? _e : 'new'}`);
                const upsertResult = await transactionManager.upsert(WorkflowEntity_1.WorkflowEntity, { ...importedWorkflow }, ['id']);
                if (((_f = upsertResult === null || upsertResult === void 0 ? void 0 : upsertResult.identifiers) === null || _f === void 0 ? void 0 : _f.length) !== 1) {
                    throw new Error(`Failed to upsert workflow ${(_g = importedWorkflow.id) !== null && _g !== void 0 ? _g : 'new'}`);
                }
                const upsertedWorkflowId = upsertResult.identifiers[0].id;
                await transactionManager.upsert(SharedWorkflow_1.SharedWorkflow, {
                    workflowId: upsertedWorkflowId,
                    userId,
                    roleId: ownerWorkflowRole.id,
                }, ['workflowId', 'userId']);
                if (existingWorkflow === null || existingWorkflow === void 0 ? void 0 : existingWorkflow.active) {
                    try {
                        n8n_workflow_1.LoggerProxy.debug(`Deactivating workflow id ${existingWorkflow.id}`);
                        await workflowRunner.remove(existingWorkflow.id);
                        n8n_workflow_1.LoggerProxy.debug(`Reactivating workflow id ${existingWorkflow.id}`);
                        await workflowRunner.add(existingWorkflow.id, 'activate');
                    }
                    catch (error) {
                        n8n_workflow_1.LoggerProxy.error(`Failed to activate workflow ${existingWorkflow.id}`, error);
                    }
                }
                return {
                    id: (_h = importedWorkflow.id) !== null && _h !== void 0 ? _h : 'unknown',
                    name: file,
                };
            }));
        });
        return importWorkflowsResult;
    }
    async importFromWorkFolder(options) {
        try {
            const importedVariables = await this.importVariablesFromFile(options.variables);
            const importedCredentials = await this.importCredentialsFromFiles(options.userId);
            const importWorkflows = await this.importWorkflowsFromFiles(options.userId);
            const importTags = await this.importTagsFromFile();
            return {
                variables: importedVariables,
                credentials: importedCredentials,
                workflows: importWorkflows,
                tags: importTags,
            };
        }
        catch (error) {
            throw Error(`Failed to import workflows from work folder: ${error.message}`);
        }
    }
};
exports.SourceControlImportService = SourceControlImportService = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [])
], SourceControlImportService);
//# sourceMappingURL=sourceControlImport.service.ee.js.map